//////////////////// UniqueURL //////////////////////////////////////////////////
//
// UniqueURL creates a unique URL for get requests that only care about the
// first few characters, the remaining part of the URL is the time in seconds
// since 1970 and a "fetchCounter", so that we never request the same URL twice.
//

function UniqueURL( baseUrl ) {
  this.fetchCounter = 0;
  this.baseUrl = baseUrl;
  this.enabled = true;
}

UniqueURL.prototype.get = function() {
  var d = new Date();
  if ( this.enabled ) {
    this.fetchCounter = this.fetchCounter + 1;
    return( this.baseUrl + d.getTime() + "" + this.fetchCounter );
  } else {
    return( this.baseUrl );
  }
}

UniqueURL.prototype.disable = function() {
  this.enabled = false;
}

UniqueURL.prototype.enable = function() { 
  this.enabled = true;
}

////////////////// NeatTools_XServer ///////////////////////////////////////////
//
// NeatTools_XServer: interfaces to neattools XServer external module
//

loadedValues = new Array();  // for some reason my NeatTools_Xserver
// is not behaving as an object..., constructor and query work, but 
// when "loadValues" is called it is as if constructor was NOT called.
//

function NeatTools_XServer( urlGenerator ) {
  this.urlGenerator = urlGenerator;
  this.numberMaps = 0;
  try {
    this.xmlhttp = new XMLHttpRequest();
  } catch (e) {
    this.xmlhttp=false
  }
  if (!this.xmlhttp) {
    try {
      this.xmlhttp=new ActiveXObject("Msxml2.XMLHTTP")
    } catch (e) {
      try {
        this.xmlhttp=new ActiveXObject("Microsoft.XMLHTTP")
      } catch (E) {
        this.xmlhttp=false
      }
    }
  }
}

var numberMapsExpectedInResponse = 0;
NeatTools_XServer.prototype.query = function( numberMaps ) {
  numberMapsExpectedInResponse = numberMaps;
  try {
    this.xmlhttp.open("GET", this.urlGenerator.get(),true);
    this.xmlhttp.onreadystatechange=function() {
      if (neattools.xmlhttp.readyState==4) {
        this.loadValues(neattools.xmlhttp.responseText);
     }
    }
    this.xmlhttp.send(null)

    // don't repeat if we ever get an exception
    // TEMP: never repeat! setTimeout("queryNeattools();",1000);
  } catch (e ) {
    //alert( "GET exception" );
    //for ( x in e ) {
     // alert( "exception has: " + x + ", value: " + e[x]);
   // }
  }
}


NeatTools_XServer.prototype.addValueLine = function addValueLine( s ) {
  for ( var i = 0; i < loadedValues.length; i++ ) {
    if ( this.loadedValues[i][0] == s[0] ) {
      this.loadedValues[i] = s;
      alert( "found and replaced" );
      return;
    }
  }
  this.loadedValues[ this.loadedValues.length ] = s;
}

NeatTools_XServer.prototype.loadValues = function loadValues( s ) {
  lines = s.split( '\n' );
  for ( var i = 0; i < lines.length; i++ ) {
    valueLine = lines[i].split( ',' );
    if ( valueLine.length == 10 ) {
      this.addValueLine( valueLine );
    }
  }
  // 
  // the following functions must exist on the HTML side
  //
  updateTrackingTable();
  setVehicleLocations();
}

////////////////// Map ////////////////////////////////////////////////////////
//
// The Map is a wrapper around a dynapi object
//

function Map( mapImage, viewPriority, neattoolsIndicator ) {
  this.mapImage = mapImage;
  this.viewPriority = viewPriority;
  this.neattoolsIndicator = neattoolsIndicator; // neattools uses this as identifier (implicitly by offset)
  this.hitCount = 0;
}

Map.prototype.setBounds = function( ulLatitude, ulLongitude, lrLatitude, lrLongitude ) {
  this.ul_latitude = parseFloat( ulLatitude );
  this.ul_longitude = parseFloat( ulLongitude );
  this.lr_latitude = parseFloat( lrLatitude );
  this.lr_longitude = parseFloat( lrLongitude );
  this.xRange = this.lr_longitude - this.ul_longitude;
  this.yRange = this.lr_latitude - this.ul_latitude;
}

Map.prototype.toX = function( longitude, maxX ) {
  return( Math.floor(( longitude - this.ul_longitude )/this.xRange * maxX ));
}

Map.prototype.toY = function( latitude, maxY ) {
  return( Math.floor(( latitude - this.ul_latitude )/this.yRange * maxY ));
}

Map.prototype.hasLocation = function( latitude, longitude ) {
  flat = parseFloat( latitude );
  flong = parseFloat( longitude );
  if ( this.ul_latitude > this.lr_latitude ) {
    if ( flat > this.ul_latitude ) {
      return( false );
    } else if ( flat < this.lr_latitude ) {
      return( false );
    }
  } else {
    if ( flat < this.ul_latitude ) {
      return( false );
    } else if ( flat > this.lr_latitude ) {
      return( false );
    }
  }
  if ( this.ul_longitude > this.lr_longitude ) {
    if ( flong > this.ul_longitude ) {
      return( false );
    } else if ( flong < this.lr_longitude ) {
      return( false );
    }
  } else {
    if ( flong < this.ul_longitude ) {
      return( false );
    } else if ( flong > this.lr_longitude ) {
      return( false );
    }
  }
  return( true );
}

Map.prototype.setView = function( mapView ) {
  this.mapView = mapView;
  mapView.setZIndex( this.viewPriority );
  mapView.setVisible( false );
  mapView.fadeState = 1;
}


////////////////// Vehicle /////////////////////////////////////////////////////
//
// Vehicle is a moving image on top of a map image
//
function Vehicle( dynImage ) {
  this.dynImage = dynImage;
  this.imageUrl = "";
  this.tracking = false;
  this.view = null;
}

Vehicle.prototype.update = function( timlist ) {
  this.vid = timlist[0];
  this.latitude = timlist[1];
  this.longitude = timlist[2];
  this.timestamp = timlist[3];
  this.statusIndicator = timlist[4];
  this.imageURL = timlist[5];

  this.ulLongitude = timlist[6];
  this.ulLatitude = timlist[7];
  this.lrLongitude = timlist[8];
  this.lrLatitude = timlist[9];
}

Vehicle.prototype.setTracking = function( val ) {
  this.tracking = val;
}

Vehicle.prototype.setImageUrl = function( imageUrl ) {
  this.imageUrl = imageUrl;
}

Vehicle.prototype.setView = function( v ) {
  this.view = v;
}

Vehicle.prototype.getImage = function() {
  return( this.dynImage );
}

Vehicle.prototype.getView = function() {
  return( this.view );
}


////////////////// GPS_rovers ////////////////////////////////////////////////
//
// GPS_rovers:  manages the set of Vehicles (moving images) associated with 
//  changing GPS locations
//

function GPS_rovers( baseUrl, dynapi ) {
  this.baseUrl = baseUrl;
  this.dynapiFunctions = dynapi.functions;
  this.dynapiDocument = dynapi.document;
  this.vehicleList = new Array();
}

GPS_rovers.prototype.lookupVehicleById = function( id ) {
  return 0;
}

var imageList = new Array();
imageList[0] = 'homer.gif';
imageList[1] = 'snoopy.gif';
GPS_rovers.prototype.addVehicleTim = function( timlist ) {
  var vehicle = this.addVehicle( imageList[ this.vehicleList.length % 2 ], 10 );
  vehicle.update( timlist );
  vehicleImage = vehicle.getImage();
  v = maparea.addChild(new DynLayer(vehicleImage.getHTML(),-vehicleImage.width,-vehicleImage.height,vehicleImage.width,vehicleImage.height));
  v.setZIndex(10);
  vehicle.setView( v );
}

GPS_rovers.prototype.addVehicle = function( vehicleFileName, viewPriority ) {
  var dynImage = this.dynapiFunctions.getImage( this.baseUrl + vehicleFileName );
  var vehicle = this.vehicleList[ this.vehicleList.length ] = new Vehicle( dynImage ); 
  vehicle.setImageUrl( this.baseUrl + vehicleFileName );
  return vehicle;
}

GPS_rovers.prototype.getNumberVehicles = function() {
  return( this.vehicleList.length );
}

GPS_rovers.prototype.getVehicle = function( offset ) {
  return( this.vehicleList[ offset] );
}


//
// load table displaying a vehicle, assumes columns are:
//
//   icon, latitude, longitude
//
GPS_rovers.prototype.loadDisplay = function( table ) {
  numberVehicles = this.vehicleList.length;
  for ( var i = 0; i < numberVehicles; i++ ) {
    row = table.insertRow( table.rows.length );
    cell = row.insertCell( 0 );
    cell.innerHTML = "<input type=\"CHECKBOX\" onclick=\"trackVehicle(this, '" + i + "');\">";
    cell = row.insertCell( 1 );
    vehicle = this.vehicleList[ i ];
    //cell.innerHTML = "<img src=\"" + vehicle.imageUrl + "\">";
    cell.innerHTML = "&nbsp;" + vehicle.vid;
    cell = row.insertCell( 2 );
    cell.id = "n" + (i+1) + "lat";
    cell.innerHTML = "&nbsp;" + vehicle.latitude;
    cell = row.insertCell( 3 );
    cell.id = "n" + (i+1) + "long";
    cell.innerHTML = "&nbsp;" + vehicle.longitude;
  }
}

////////////////// MapManager /////////////////////////////////////////////////
//
// MapManager handles the set of images that are span a fixed range of 
// GPS locations
//

function MapManager( baseUrl, dynapi ) {
  this.baseUrl = baseUrl;
  this.dynapiFunctions = dynapi.functions;
  this.dynapiDocument = dynapi.document;
  this.mapList = new Array();
  
}

MapManager.prototype.numberMatches = function( latitude, longitude ) {
  var result = 0;
  for ( var i = 0; i < this.mapList.length; i++ ) {
    var map = this.mapList[i];
    if ( map.hasLocation( latitude, longitude )) {
      result += 1;
    }
  }
  return( result );
}

MapManager.prototype.initMapSet = function() {
  for ( var i = 0; i < this.mapList.length; i++ ) {
    var map = this.mapList[i];
    map.hitCount = 0;
  }
}

MapManager.prototype.getBestView = function() {
  var maxHit = 0;
  var selectedMap = 0;
  for ( var i = 0; i < this.mapList.length; i++ ) {
    var map = this.mapList[i];
    if ( map.hitCount > maxHit ) {
      selectedMap = i;
    }
  }
  this.bestMap = this.mapList[ selectedMap ];
  return( this.mapList[ selectedMap ].mapView );
}

MapManager.prototype.trackMatches = function( latitude, longitude ) {
  for ( var i = 0; i < this.mapList.length; i++ ) {
    var map = this.mapList[i];
    if ( map.hasLocation( latitude, longitude )) {
      map.hitCount += 1;
    }
  }
  for ( var i = 0; i < this.mapList.length; i++ ) {
    var map = this.mapList[i];
    if ( map.hitCount > 0 ) {
//      alert( "map " + i + " has hitCount " + map.hitCount );
    }
  }
}

MapManager.prototype.numberMaps = function() {
  return( this.mapList.length );
}


MapManager.prototype.addMap = function( mapFileName, viewPriority, neattoolsIndicator ) {
  var dynImage = this.dynapiFunctions.getImage( this.baseUrl + mapFileName );
  var map = this.mapList[ this.mapList.length ] = new Map( dynImage, viewPriority, neattoolsIndicator );
  return map;
}


MapManager.prototype.getView = function( neattoolsOffset ) {
  for ( var i = 0; i < this.mapList.length; i++ ) {
    m = this.mapList[i];
    if ( m.neattoolsIndicator == neattoolsOffset ) {
      return( m.mapView );
    }
  }
  return 0;
}
